==> webbase-5.9.0.tar.gz: webbase-5.9.0/hooks/WebbaseHookMifluz.cc <==
/*
 *   Copyright (C) 1997, 1998
 *   	Free Software Foundation, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by the
 *   Free Software Foundation; either version 2, or (at your option) any
 *   later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#ifdef HAVE_MIFLUZ_H

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <unistd.h>

#include <strshift.h>
#include <rm.h>
#include <salloc.h>
#include <hash.h>
#include <md5str.h>
#include <isomap.h>
#include <mkdirp.h>

#include <sqlutil.h>
#include <html_content.h>
#include <WebbaseHookMifluz.h>

#define POS_Word     0
#define POS_Server   1
#define POS_URL      2
#define POS_Tag      3
#define POS_Location 4 


extern "C" {
void* hooksmifluz_init();
}

//
// Dynamic loading symbol and initialization function
//
void* hooksmifluz_init()
{
  return (void*)new WebbaseHookMifluz();
}

static int html_content_collect(int info, html_content_parsed_t* parsed, void* data);

WebbaseHookMifluz::WebbaseHookMifluz()
{
  static ConfigDefaults defaults[] = {
    { "wordlist_extend", "true", 0 },
    { "minimum_word_length", "1", 0 },
    { "maximum_word_length", "25", 0 },
    { "wordlist_cache_size", "100000000", 0 }, // 10Mb
    { "wordlist_page_size", "4096", 0 },     // 32Kb
    { "wordlist_compress", "1", 0 },
    { "wordlist_wordkey_description","nfields: 5/URL 32 2/Location 16 4/Server 19 1/Tag 5 3/Word 0 0"},
#if 0
    { "wordlist_monitor","true"},
    { "wordlist_monitor_period","10.0"},
    { "wordlist_monitor_input","true" },
    { "wordlist_compress_debug", "0"},
#endif
    { 0, 0, 0 }
  };

  config = WordContext::Initialize(defaults);
  wordRef = new WordReference();
}

WebbaseHookMifluz::~WebbaseHookMifluz()
{
  Close();
  delete wordRef;
  delete config;
}

struct option* WebbaseHookMifluz::Options()
{
  static struct option long_options[] = {
    {"verbose_hooks", 0, 0, 0},
    {"hook_cache_size", 1, 0, 0},
    {"hook_page_size", 1, 0, 0},
    {"hook_compress", 0, 0, 0},
    {0, 0, 0, 0}
  };

  return long_options;
}

int WebbaseHookMifluz::HandleOption(struct option* option)
{
  int found = 1;

  if(!WebbaseHook::HandleOption(option)) {
    if(!strcmp(option->name, "hook_cache_size")) {
      config->Add("wordlist_cache_size", optarg);
    } else if(!strcmp(option->name, "hook_page_size")) {
      config->Add("wordlist_page_size", optarg);
    } else if(!strcmp(option->name, "hook_compress")) {
      config->Add("wordlist_compress", "1");
    } else {
      found = 0;
    }
  }

  return found;
}

int WebbaseHookMifluz::Init(webbase_t* base)
{
  WebbaseHook::Init(base);
  Open();
  return 1;
}

int WebbaseHookMifluz::Ok(webbase_url_t* webbase_url)
{
  if((webbase_url->w_info & WEBBASE_URL_INFO_CONTENT) &&
     webbase_url->w_content_length > 0 &&
     !webbase_url_robots_p(webbase_url))
    return 1;
  else
    return 0;
}

int WebbaseHookMifluz::Insert(webbase_url_t* webbase_url)
{
  WordKey& key = wordRef->Key();
  /*
   * Prepare the insertion context
   */
  wordRef->Clear();
  key.Set(POS_URL,webbase_url->w_rowid);
  key.Set(POS_Server,server_id);
  
  /*
   * Invoke the HTML parser
   */
  html_content_t walk;

  html_content_reset(&walk);
  walk.parser.info = HTML_SOURCE_FILENAME;
  char* path = uri_furi_string(webbase_url->w_url, strlen(webbase_url->w_url), URI_FURI_REAL_PATH);
  walk.parser.source = path;
  walk.parser.source_length = strlen(path);
  walk.content_callback = html_content_collect;
  walk.content_data = (void*)this;

  return html_content_parse(&walk);
}

int WebbaseHookMifluz::InsertWatch()
{
  return 0;
}

int WebbaseHookMifluz::Update(webbase_url_t*)
{
  return 0;
}

int WebbaseHookMifluz::Delete(webbase_url_t*)
{
  return 0;
}

int WebbaseHookMifluz::DeleteId(int, char*, int)
{
  return 0;
}

int WebbaseHookMifluz::RehookStart()
{
  return 0;
}

int WebbaseHookMifluz::RehookEnd()
{
  return 0;
}

int WebbaseHookMifluz::RebuildStart()
{
  WebbaseHook::RebuildStart();
  //
  // Reset the index itself
  //
  char* database = Database();
  Close();
  unlink(database);
  free(database);
  return Open();
}

int WebbaseHookMifluz::RebuildEnd()
{
  return 0;
}

void WebbaseHookMifluz::InfoSet(char*)
{
}

void WebbaseHookMifluz::Server(int server_arg)
{
  server_id = server_arg;
}

void WebbaseHookMifluz::End()
{
}

//
// Mifluz implementation
//

//
// insert helpers
//
/*
 * C callback redirecting to WebbaseHookMifluz member function.
 */
static int html_content_collect(int info, html_content_parsed_t* parsed, void* data)
{
  WebbaseHookMifluz* mifluz = (WebbaseHookMifluz*)data;
  return mifluz->InsertChunk(info, parsed);
}

int WebbaseHookMifluz::InsertChunk(int info, html_content_parsed_t* parsed)
{
  int ret = 1;
  char* buffer = parsed->buffer0;
  int buffer_length = parsed->buffer0_length;
  WordKey& key = wordRef->Key();

  switch(info) {

  case HTML_CONTENT_META:
    {
      buffer = parsed->buffer1;
      buffer_length = parsed->buffer1_length;

      if(!strncasecmp(parsed->buffer0, "keyword", 7)) {
	key.Set(POS_Tag,WEBBASE_URL_TAG_KEY);
      } else if(!strncasecmp(parsed->buffer0, "description", 11)) {
	key.Set(POS_Tag,WEBBASE_URL_TAG_DESCRIPTION);
      } else {
	buffer = 0;
	buffer_length = 0;
      }
    }
    break;

  case HTML_CONTENT_TITLE:
    key.Set(POS_Tag,WEBBASE_URL_TAG_TITLE);
    break;

  case HTML_CONTENT_TEXT:
    key.Set(POS_Tag,WEBBASE_URL_TAG_BODY);
    break;
  }

  if(buffer)
    ret = InsertContent(buffer, buffer_length);
  
  return ret;
}  

int WebbaseHookMifluz::InsertContent(const char* buffer, int buffer_length)
{
  const WordType& wtype = inv->GetWordType();
  const unsigned char* p;
  const unsigned char* word_start;
  WordKey& key = wordRef->Key();

  unsigned char* buffer_tmp = (unsigned char*)malloc(buffer_length + 1);
  memcpy(buffer_tmp, buffer, buffer_length);
  buffer_tmp[buffer_length] = '\0';
  p = word_start = buffer_tmp;
  
  unaccent(buffer_tmp, buffer_length);
  while(p - buffer_tmp < buffer_length) {
    while(word_start - buffer_tmp < buffer_length && !wtype.IsChar(*word_start))
      word_start++;
    p = word_start;
    while(p - buffer_tmp < buffer_length && wtype.IsChar(*p))
      p++;
    if(word_start < p) {
      key.Set(POS_Location,key[POS_Location]+1);
      key.SetWord(String((char*)word_start, p - word_start));
      if(verbose) cerr << "WebbaseHookMifluz::InsertContent: " << *wordRef << endl;
      inv->Insert(*wordRef);
    }
    word_start = p;
  }

  free((void *)buffer_tmp);
  return 1;
}

//
// Return the pathname of the database. The caller must free it.
//
char* WebbaseHookMifluz::Database() const
{
  char* database;
  /*
   * Open mifluz session
   */
  if((database = getenv("FULLTEXTDB")) == 0) {
    char* ecilaroot = getenv("ECILAROOT");
    if(!ecilaroot)
      ecilaroot = ".";
    database = smalloc(strlen(ecilaroot) + strlen(base->name) * 2 + 128);
    sprintf(database, "%s/db/M%s", ecilaroot, base->name);
    mkdirp(database, 0777);
    sprintf(database, "%s/db/M%s/%s", ecilaroot, base->name, base->name);
  } else {
    database = strdup(database);
  }

  return database;
}

int WebbaseHookMifluz::Open()
{
  if(!inv) {
    inv = new WordList(*config);

    char* database = Database();

    int ret = inv->Open(database, O_RDWR);

    free(database);

    return ret;
  } else {
    return 0;
  }
}

int WebbaseHookMifluz::Close()
{
  if(inv) {
    delete inv;
    inv = 0;
  }
  return 1;
}

#endif /* HAVE_MIFLUZ_H */

==> webbase-5.10.0.tar.gz: webbase-5.10.0/hooks/WebbaseHookMifluz.cc <==
/*
 *   Copyright (C) 1997, 1998
 *   	Free Software Foundation, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by the
 *   Free Software Foundation; either version 2, or (at your option) any
 *   later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#ifdef HAVE_MIFLUZ_H

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <unistd.h>

#include <strshift.h>
#include <rm.h>
#include <salloc.h>
#include <hash.h>
#include <md5str.h>
#include <isomap.h>
#include <mkdirp.h>

#include <sqlutil.h>
#include <html_content.h>
#include <WebbaseHookMifluz.h>

#define POS_Word     0
#define POS_Server   1
#define POS_URL      2
#define POS_Tag      3
#define POS_Location 4 


extern "C" {
void* hooksmifluz_init();
}

//
// Dynamic loading symbol and initialization function
//
void* hooksmifluz_init()
{
  return (void*)new WebbaseHookMifluz();
}

static int html_content_collect(int info, html_content_parsed_t* parsed, void* data);

WebbaseHookMifluz::WebbaseHookMifluz()
{
  static ConfigDefaults defaults[] = {
    { "wordlist_extend", "true", 0 },
    { "minimum_word_length", "1", 0 },
    { "maximum_word_length", "25", 0 },
    { "wordlist_cache_size", "100000000", 0 }, // 10Mb
    { "wordlist_page_size", "8192", 0 },     // 8Kb
    { "wordlist_compress", "1", 0 },
    { "wordlist_wordkey_description","Word/Server 19/URL 32/Tag 5/Location 16"},
    { 0, 0, 0 }
  };

  config = WordContext::Initialize(defaults);
  wordRef = new WordReference();
}

WebbaseHookMifluz::~WebbaseHookMifluz()
{
  Close();
  delete wordRef;
  delete config;
}

struct option* WebbaseHookMifluz::Options()
{
  static struct option long_options[] = {
    {"verbose_hooks", 0, 0, 0},
    {"hook_cache_size", 1, 0, 0},
    {"hook_page_size", 1, 0, 0},
    {"hook_compress", 0, 0, 0},
    {0, 0, 0, 0}
  };

  return long_options;
}

int WebbaseHookMifluz::HandleOption(struct option* option)
{
  int found = 1;

  if(!WebbaseHook::HandleOption(option)) {
    if(!strcmp(option->name, "hook_cache_size")) {
      config->Add("wordlist_cache_size", optarg);
    } else if(!strcmp(option->name, "hook_page_size")) {
      config->Add("wordlist_page_size", optarg);
    } else if(!strcmp(option->name, "hook_compress")) {
      config->Add("wordlist_compress", "1");
    } else {
      found = 0;
    }
  }

  return found;
}

int WebbaseHookMifluz::Init(webbase_t* base)
{
  WebbaseHook::Init(base);
  Open(O_RDWR);
  return 1;
}

int WebbaseHookMifluz::Ok(webbase_url_t* webbase_url)
{
  if((webbase_url->w_info & WEBBASE_URL_INFO_CONTENT) &&
     webbase_url->w_content_length > 0 &&
     !webbase_url_robots_p(webbase_url))
    return 1;
  else
    return 0;
}

int WebbaseHookMifluz::Insert(webbase_url_t* webbase_url)
{
  WordKey& key = wordRef->Key();
  /*
   * Prepare the insertion context
   */
  wordRef->Clear();
  key.Set(POS_URL,webbase_url->w_rowid);
  key.Set(POS_Server,server_id);
  
  /*
   * Invoke the HTML parser
   */
  html_content_t walk;

  html_content_reset(&walk);
  walk.parser.info = HTML_SOURCE_FILENAME;
  char* path = uri_furi_string(webbase_url->w_url, strlen(webbase_url->w_url), URI_FURI_REAL_PATH);
  walk.parser.source = path;
  walk.parser.source_length = strlen(path);
  walk.content_callback = html_content_collect;
  walk.content_data = (void*)this;

  return html_content_parse(&walk);
}

int WebbaseHookMifluz::InsertWatch()
{
  return 0;
}

int WebbaseHookMifluz::Update(webbase_url_t*)
{
  return 0;
}

int WebbaseHookMifluz::Delete(webbase_url_t*)
{
  return 0;
}

int WebbaseHookMifluz::DeleteId(int, char*, int)
{
  return 0;
}

int WebbaseHookMifluz::RehookStart()
{
  return 0;
}

int WebbaseHookMifluz::RehookEnd()
{
  return 0;
}

int WebbaseHookMifluz::RebuildStart()
{
  WebbaseHook::RebuildStart();
  //
  // Reset the index itself
  //
  Close();
  WordContext::Initialize(*config);
  return Open(O_RDWR|O_TRUNC);
}

int WebbaseHookMifluz::RebuildEnd()
{
  Close();
  return 0;
}

void WebbaseHookMifluz::InfoSet(char*)
{
}

void WebbaseHookMifluz::Server(int server_arg)
{
  server_id = server_arg;
}

void WebbaseHookMifluz::End()
{
}

//
// Mifluz implementation
//

//
// insert helpers
//
/*
 * C callback redirecting to WebbaseHookMifluz member function.
 */
static int html_content_collect(int info, html_content_parsed_t* parsed, void* data)
{
  WebbaseHookMifluz* mifluz = (WebbaseHookMifluz*)data;
  return mifluz->InsertChunk(info, parsed);
}

int WebbaseHookMifluz::InsertChunk(int info, html_content_parsed_t* parsed)
{
  int ret = 1;
  char* buffer = parsed->buffer0;
  int buffer_length = parsed->buffer0_length;
  WordKey& key = wordRef->Key();

  switch(info) {

  case HTML_CONTENT_META:
    {
      buffer = parsed->buffer1;
      buffer_length = parsed->buffer1_length;

      if(!strncasecmp(parsed->buffer0, "keyword", 7)) {
	key.Set(POS_Tag,WEBBASE_URL_TAG_KEY);
      } else if(!strncasecmp(parsed->buffer0, "description", 11)) {
	key.Set(POS_Tag,WEBBASE_URL_TAG_DESCRIPTION);
      } else {
	buffer = 0;
	buffer_length = 0;
      }
    }
    break;

  case HTML_CONTENT_TITLE:
    key.Set(POS_Tag,WEBBASE_URL_TAG_TITLE);
    break;

  case HTML_CONTENT_TEXT:
    key.Set(POS_Tag,WEBBASE_URL_TAG_BODY);
    break;
  }

  if(buffer)
    ret = InsertContent(buffer, buffer_length);
  
  return ret;
}  

int WebbaseHookMifluz::InsertContent(const char* buffer, int buffer_length)
{
  const WordType& wtype = inv->GetWordType();
  const unsigned char* p;
  const unsigned char* word_start;
  WordKey& key = wordRef->Key();

  unsigned char* buffer_tmp = (unsigned char*)malloc(buffer_length + 1);
  memcpy(buffer_tmp, buffer, buffer_length);
  buffer_tmp[buffer_length] = '\0';
  p = word_start = buffer_tmp;
  
  unaccent(buffer_tmp, buffer_length);
  while(p - buffer_tmp < buffer_length) {
    while(word_start - buffer_tmp < buffer_length && !wtype.IsChar(*word_start))
      word_start++;
    p = word_start;
    while(p - buffer_tmp < buffer_length && wtype.IsChar(*p))
      p++;
    if(word_start < p) {
      key.Set(POS_Location,key[POS_Location]+1);
      key.SetWord(String((char*)word_start, p - word_start));
      if(verbose) fprintf(stderr, "WebbaseHookMifluz::InsertContent: %s\n", (char*)wordRef->Get());
      inv->Insert(*wordRef);
    }
    word_start = p;
  }

  free((void *)buffer_tmp);
  return 1;
}

//
// Return the pathname of the database. The caller must free it.
//
char* WebbaseHookMifluz::Database() const
{
  char* database;
  /*
   * Open mifluz session
   */
  if((database = getenv("FULLTEXTDB")) == 0) {
    char* ecilaroot = getenv("ECILAROOT");
    if(!ecilaroot)
      ecilaroot = ".";
    database = smalloc(strlen(ecilaroot) + strlen(base->name) * 2 + 128);
    sprintf(database, "%s/db/M%s", ecilaroot, base->name);
    mkdirp(database, 0777);
    sprintf(database, "%s/db/M%s/%s", ecilaroot, base->name, base->name);
  } else {
    database = strdup(database);
  }

  return database;
}

int WebbaseHookMifluz::Open(int flags)
{
  if(!inv) {
    inv = new WordList(*config);

    char* database = Database();

    int ret = inv->Open(database, flags);

    free(database);

    return ret;
  } else {
    return 0;
  }
}

int WebbaseHookMifluz::Close()
{
  if(inv) {
    delete inv;
    inv = 0;
  }
  return 1;
}

#endif /* HAVE_MIFLUZ_H */
